*******************************************************************************
* Project:	ultraSTE
* Module:	Layer3
* Code:		ultra
* Created:	20.05.2022
* Notes:	Routines for Layer 3 handling
*		layer 3 is more or less a sprite which can be used only in
*		the new modes. low- and midres are not supported.
*		On real hardware it works but i didn't included it in Steem.
*		
*		the layer data is located in fpga memory and must be transfered
*		before display. a maximum of 16kb is possible
*		
*		the data of the fpga is word based which means you can only
*		write (read is not possible) from atari with words to the memory.
*		you can modify the sprite of course to create for example a scroller.
*		the data format for the sprite is chunky based one byte one pixel.
*		for the colors USColorTable4R is used.
*
*		width and height is flexible for the sprites as long as it fits
*		into the 16kb. for example a 160*100 sprite needs 16kb and fits
*		nicely into the reserved memory.
*
*		the sprite display is always at 640*400 resolution (on steem 640*200)
*		because of that you can scale the sprite by 1,2,4,8,16,32,64,128
*		pixels to fill the screen. Scaling can be set for x and y individually
*		positioning of the sprite is still in 640*400 coordinates.
*		it's of course possible to multiplex the sprites.
*		During multiplex you can use the memory offset to display a different
*		block of the FPGA memory.
*		If you write to a layer3 register the current reg is incred
*		automatically. Thia speeds up multiplexing.
*
*		Mirrowing in x or y can be set individually.
*		
*		If the repeat mode is active the sprite does not end when the size in
*		x and y is reached. Its repeated.
*
*		The Layer3 can be displayed in various modes:
*		-add: rgb values of layer 3 and normal output are added
*		-sub: rgb values of layer 3 are subbed from the normal output
*		-overlay: a color of $000000 is transparent. layer 3 overlays
*		 the normal output
*		-xor mode: xors layer3 with the outputted rgb values
*
*		The brightness of the Layer3 can be of course set by a register.
*
*		Have a look to the USDefines.s there are alot registers / enums defined
* Modified:
* Changes:
*
* (c) 2022 cream
*******************************************************************************

USLayerSetOptions:
; d0 -> options
; bit 0 	= enable layer 3
; bit 1 	= (unused)
; bit 2-3 	= mode
;		  00 add layer 3 to normal output
;		  01 sub layer 3 to normal output
;		  10 overlay. layer 3 overlays  normal output
;		  11 layer 3 xors normal overlay 
; bit 4 	= layer 3 mirrow x 
; bit 5 	= layer 3 mirrow y
; bit 6 	= (unused)
; bit 7 	= (unused)
; bit 8 	= layer 3 repeat. drawing of the sprite does not stop after reaching
;		  width or height. repeats until border
; bit 9 	= (unused)
; bit 10-12	= layer 3 scale 
;		  %000 = 1, %001 = 2, %010 = 4, %011 = 8
;		  %100 = 16, %101 = 32, %110 = 64, %111 = 128
;		  keep in mind the sprite is displayed in 640x400
;		  so you maybe need a scale

	USWrReg	d0,USLayer3Options
	rts

*//****************************************************************************
USLayer3SetPos:
; d0 -> x
; d1 -> y
;
; sets the x/y position of the layer 3
; values for x and y are in 640*400 coordinates
; 0,0 is upper left corner of the st screen.
; x and y can be negative but max -width*scale and -height*scale
; overscan is not supported currently

	USWrReg	d0,USLayer3XPos
	USWrReg	d1,USLayer3YPos
	rts

*//****************************************************************************
USLayer3SetMemOffset:
; d0 -> offset
;
; sets the memory offset of the layer 3
; this is for example if you multiplex the sprite into 2 sprites
; upper and lower one to have different graphics.

	USWrReg	d0,USLayer3MemOffset
	rts

*//****************************************************************************
USLayer3SetFadeToBlack:
; d0 -> fade
;
; fades all colors of layer 3 to black
; $00 black
; $ff full brightness

	USWrReg	d0,USFadeToBlack3
	rts

*//****************************************************************************
USLayer3SetSize:
; d0 -> width
; d1 -> heigth
;
; sets the size of the layer 3

	USWrReg	d0,USLayer3XSize
	USWrReg	d1,USLayer3YSize
	rts

*//****************************************************************************
USLayer3SendData:
; d0 -> fpga memory position (max. 8000)
;	keep in mind the memory is word based! you need to do position/2!
; d1 -> length (max. 8000)
;	keep in mind the memory is word based! you need to do length/2!
; a0 -> memory pointer to the data
;
; transfers the graphics data to fpga memory
; graphics data is chunky based 1 byte one pixel
; but it's only possible to transfer a word .b is not allowed!
; ok maybe not the fastest transfer but works ;)
;

	USWrReg	d0,USSetLayer3Adr

	moveq	#0,d0
.copy
	move	(a0)+,USRegData.w
	addq	#1,d0
	cmp	d0,d1
	bne.s	.copy
	rts

